import cv2
import torch
from models.experimental import attempt_load
from utils.datasets import prepare_img
from utils.general import check_img_size, non_max_suppression
from utils.torch_utils import TracedModel, select_device, time_synchronized


class Detector:
    def __init__(self, weights="yolov7.pt", img_size=320, device=""):
        self.device = select_device(device)
        self.half = self.device.type != "cpu"  # half precision only supported on CUDA

        # Load model
        model = attempt_load(weights, map_location=self.device)  # load FP32 model
        self.stride = int(model.stride.max())  # model stride
        self.imgsz = check_img_size(img_size, s=self.stride)  # check img_size
        self.model_image_shape = torch.Size([int(0.6 * self.imgsz), self.imgsz])
        self.model = TracedModel(model, self.device, self.imgsz)

        if self.half:
            self.model.half()  # to FP16

        # Warm-up
        if self.device.type != "cpu":
            self.model(torch.zeros(3, 3, self.imgsz, self.imgsz).to(self.device).type_as(next(model.parameters())))

    def detect(self, frames):
        prepared_frames = []
        # preprocess all frames and add them to a list
        for frame in frames:
            img = prepare_img(frame, img_size=self.imgsz, stride=self.stride)
            img = torch.from_numpy(img).to(self.device)
            img = img.half() if self.half else img.float()  # uint8 to fp16/32
            img /= 255.0  # 0 - 255 to 0.0 - 1.0
            if img.ndimension() == 3:
                img = img.unsqueeze(0)
            prepared_frames.append(img)
        # Cat into one tensor and run inference on (n_frames, 3)
        prepared_frames = torch.cat(prepared_frames, 0)
        with torch.no_grad():
            pred = self.model(prepared_frames, augment=False)[0]
        # NMS
        pred = non_max_suppression(
            pred,
            0.5,
            0.45,
            classes=[2, 3, 7],
            agnostic=True,
        )


dt = Detector()
frames = [cv2.imread("img.png")]
x = 0
while True and x < 1000:
    t1 = time_synchronized()
    dt.detect(frames)
    t2 = time_synchronized()
    print(t2 - t1)
    x += 1